#!/usr/bin/env python

"A WebStack application for a PEP 241 repository."

import WebStack.Generic
import XSLForms.Resources
import XSLForms.Utils
import XSLOutput
import os

# Site map imports.

from WebStack.Resources.ResourceMap import MapResource
from WebStack.Resources.Static import DirectoryResource

# Resource classes.

class PEP241Resource(XSLForms.Resources.XSLFormsResource):

    "A resource providing repository browsing."

    resource_dir = os.path.join(os.path.split(__file__)[0], "Resources")
    encoding = "utf-8"
    template_resources = {
        "pep241" : ("pep241_template.xhtml", "pep241_output.xsl")
        }

    def respond_to_form(self, trans, form):

        """
        Respond to a request having the given transaction 'trans' and the given
        'form' information.
        """

        parameters = form.get_parameters()
        documents = form.get_documents()
        selectors = form.get_selectors()

        # Ensure the presence of a document.

        if documents.has_key("package"):
            package = documents["package"]
        else:
            package = form.new_instance("package")

        # Add and remove elements according to the selectors found.

        XSLForms.Utils.add_elements(selectors.get("add_platform"), "platform", "platforms")
        XSLForms.Utils.remove_elements(selectors.get("remove_platform"))
        XSLForms.Utils.add_elements(selectors.get("add_supported_platform"), "supported-platform", "supported-platforms")
        XSLForms.Utils.remove_elements(selectors.get("remove_supported_platform"))
        XSLForms.Utils.add_elements(selectors.get("add_keyword"), "keyword", "keywords")
        XSLForms.Utils.remove_elements(selectors.get("remove_keyword"))
        XSLForms.Utils.add_elements(selectors.get("add_author"), "author", "authors")
        XSLForms.Utils.remove_elements(selectors.get("remove_author"))
        XSLForms.Utils.add_elements(selectors.get("add_dependency"), "dependency", "dependencies")
        XSLForms.Utils.remove_elements(selectors.get("remove_dependency"))

        # Send a response according to certain parameters.
        # When exported, an XML version of the data is returned.

        if parameters.has_key("export"):
            trans.set_content_type(WebStack.Generic.ContentType("text/xml", self.encoding))
            package.toStream(trans.get_response_stream(), trans.get_response_stream_encoding())

        # When not exported, the data is transformed to produce a normal Web
        # page.

        else:

            # Transform, adding enumerations/ranges.

            categories_xsl = os.path.join(self.resource_dir, "pep241_categories.xsl")
            categories_xml = os.path.join(self.resource_dir, "pep241_categories.xml")
            proc = XSLOutput.Processor([categories_xsl], {"categories" : categories_xml})
            package = proc.get_result(package)

            # Start the response.

            trans.set_content_type(WebStack.Generic.ContentType("application/xhtml+xml", self.encoding))

            # Ensure that an output stylesheet exists.

            trans_xsl = self.prepare_output("pep241")

            # Complete the response.

            self.send_output(trans, [trans_xsl], package)

# Site map initialisation.

def get_site():

    "Return a simple Web site resource."

    # Get the main resource and the directory used by the application.

    pep241_resource = PEP241Resource()
    directory = pep241_resource.resource_dir

    # Make a simple Web site.

    resource = MapResource({
        # Static resources:
        "styles" : DirectoryResource(os.path.join(directory, "styles"), {"css" : "text/css"}),
        # Main page:
        "" : pep241_resource
        })

    return resource

# vim: tabstop=4 expandtab shiftwidth=4
